/* devicesidebar.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <netdb.h>
#include <pwd.h>
#include "devicesidebar.h"
#include "preferences.h"
#include "device.h"
#include "main.h"
#include "session.h"
#include "monitor.h"
#include "data.h"
#include "scheduler.h"
#include "parser.h"
#include "gui.h"
#include "devicedetails.h"
#include "ipaddress.h"
#include "features.h"
#include "srouter.xpm"
#include "sswitch.xpm"
#include "sunknown.xpm"

static void select_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((DeviceSideBar*)data)->select(row); }
static void unselect_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((DeviceSideBar*)data)->unselect(); }
static void add_cb(GtkWidget *w, gpointer data)
{ ((DeviceSideBar*)data)->add(); }
static void edit_cb(GtkWidget *w, gpointer data)
{ ((DeviceSideBar*)data)->edit(); }
static void delete_cb(GtkWidget *w, gpointer data)
{ ((DeviceSideBar*)data)->del(); }
static void autozoom_cb(GtkWidget *w, gpointer data)
{
	DeviceSideBar *v = (DeviceSideBar*)data;
	v->autozoom=((v->autozoom=='0')?'1':'0');
	if (v->autozoom=='1') v->main->gui->update_views();
}

DeviceSideBar::DeviceSideBar(Main *m)
{
  GtkWidget *label183;
  GtkWidget *hbox90;
  GtkWidget *button94;
  GtkWidget *scrolledwindow34;
  GtkWidget *hbox89;

	main = m;
	row = -1;

  vbox67 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox67);
  gtk_container_set_border_width (GTK_CONTAINER (vbox67), 2);

  label183 = gtk_label_new ("Device List");
  gtk_widget_show (label183);
  gtk_box_pack_start (GTK_BOX (vbox67), label183, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label183), 0, 0.5);

  hbox90 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox90);
  gtk_box_pack_start (GTK_BOX (vbox67), hbox90, FALSE, FALSE, 0);

  entry42 = gtk_entry_new ();
  gtk_widget_show (entry42);
  gtk_box_pack_start (GTK_BOX (hbox90), entry42, TRUE, TRUE, 0);
  gtk_widget_set_usize (entry42, 1, -2);
	gtk_signal_connect(GTK_OBJECT(entry42), "activate",
		GTK_SIGNAL_FUNC(add_cb), this);

  button94 = gtk_button_new_with_label ("Add");
  gtk_widget_show (button94);
  gtk_box_pack_start (GTK_BOX (hbox90), button94, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button94), "clicked",
		GTK_SIGNAL_FUNC (add_cb), this);

  scrolledwindow34 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow34);
  gtk_box_pack_start (GTK_BOX (vbox67), scrolledwindow34, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow34),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist31 = gtk_clist_new (1);
  gtk_widget_show (clist31);
  gtk_container_add (GTK_CONTAINER (scrolledwindow34), clist31);
  gtk_clist_set_column_width (GTK_CLIST (clist31), 0, 16);
  gtk_clist_column_titles_hide (GTK_CLIST (clist31));
	gtk_signal_connect(GTK_OBJECT(clist31), "select_row",
		GTK_SIGNAL_FUNC(select_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist31), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_cb), this);

  hbox89 = gtk_hbox_new (TRUE, 4);
  gtk_widget_show (hbox89);
  gtk_box_pack_start (GTK_BOX (vbox67), hbox89, FALSE, TRUE, 0);

  button95 = gtk_button_new_with_label ("Edit");
  gtk_widget_show (button95);
  gtk_box_pack_start (GTK_BOX (hbox89), button95, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button95), "clicked",
		GTK_SIGNAL_FUNC (edit_cb), this);

  button96 = gtk_button_new_with_label ("Delete");
	gtk_widget_show (button96);
  gtk_box_pack_start (GTK_BOX (hbox89), button96, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button96), "clicked",
		GTK_SIGNAL_FUNC (delete_cb), this);

  checkbutton14 = gtk_check_button_new_with_label ("Auto Zoom");
  gtk_widget_show (checkbutton14);
  gtk_box_pack_start (GTK_BOX (vbox67), checkbutton14, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (checkbutton14), "clicked",
		GTK_SIGNAL_FUNC (autozoom_cb), this);

	autozoom = '0';
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton14),
		(autozoom=='1')?TRUE:FALSE);
}

DeviceSideBar::~DeviceSideBar()
{
}

void DeviceSideBar::select(int r)
{
	Device *d;
	row = r;
	gtk_widget_set_sensitive(button96, TRUE);
	gtk_widget_set_sensitive(button95, TRUE);
	d = (Device*) gtk_clist_get_row_data(GTK_CLIST(clist31), row);
	main->session->data->select('0');
	d->selected = '1';
	if (autozoom=='1') main->gui->monitor->viewer->fitToSelected();
	main->gui->update_views();
}

void DeviceSideBar::unselect()
{
	main->session->data->select('0');
	gtk_widget_set_sensitive(button96, FALSE);
	gtk_widget_set_sensitive(button95, FALSE);
	row = -1;
	if (autozoom=='1') main->gui->monitor->viewer->fitToWindow();
	main->gui->update_views();
}

void DeviceSideBar::add()
{
	IPAddress tip;
	gchar *buf, *csl;
	struct hostent *h;

	buf = gtk_entry_get_text(GTK_ENTRY(entry42));
	gtk_entry_set_text(GTK_ENTRY(entry42), "");
	// convert name/ip into ip
	if ((h=gethostbyname(buf)) == NULL)
	{
		printf("Illegal host entry\n");
		return;
	}
	tip.setstring(inet_ntoa(*((struct in_addr *)h->h_addr)));

	main->parser->create_placeholder(tip.getstring());

	if (!main->session->devicebit)
	{
		main->session->devicebit=TRUE;
		main->setTitle();
	}
	if (!tip.zero())
	{
		csl = get_community_list_string();
		buf = g_strjoin(" ", tip.getstring(), csl, NULL);
		g_free(csl);
		main->scheduler->addTask(buf, 1, 1, 0);
		g_free(buf);
	}
}

void DeviceSideBar::edit()
{
	main->gui->devicedetails->device = NULL;
	if (row!=-1)
		main->gui->devicedetails->device =
			(Device*) gtk_clist_get_row_data(GTK_CLIST(clist31), row);
	main->gui->devicedetails->show();
}

void DeviceSideBar::del()
{
	Device *d;

	if (row!=-1)
	{
		d = (Device*) gtk_clist_get_row_data(GTK_CLIST(clist31), row);
		main->session->data->del_device(d);
		if (!main->session->devicebit)
		{
			main->session->devicebit=TRUE;
			main->setTitle();
		}
		main->gui->update_views();
		main->gui->update_lists();
	}
}

void DeviceSideBar::update()
{
	Device *d;
	Link *ln;
	gchar *data[1];
	int row;

	row=0;
	gtk_clist_freeze(GTK_CLIST (clist31));
	gtk_clist_clear(GTK_CLIST (clist31));
	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		data[0] = strdup("");
		gtk_clist_append(GTK_CLIST (clist31), data);
		delete data[0];
		if (d->type=='r')
			clist_pixtext(vbox67, GTK_CLIST(clist31), row, 0, (gchar **)srouter_xpm, d->name);
		else
		if (d->type=='s')
			clist_pixtext(vbox67, GTK_CLIST(clist31), row, 0, (gchar **)sswitch_xpm, d->name);
		else
			clist_pixtext(vbox67, GTK_CLIST(clist31), row, 0, (gchar **)sunknown_xpm, d->name);
		gtk_clist_set_row_data(GTK_CLIST(clist31), row, d);
		row++;
	}
	gtk_widget_set_sensitive(button96, FALSE);
	gtk_widget_set_sensitive(button95, FALSE);
	gtk_clist_set_sort_column (GTK_CLIST (clist31), 0);
	gtk_clist_sort (GTK_CLIST (clist31));
	gtk_clist_thaw(GTK_CLIST (clist31));
}

