/* filtermodule.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "filtermodule.h"
#include "filteritem.h"
#include "gui.h"
#include "sup.xpm"
#include "sdown.xpm"

/***************************************************************************
 Callbacks
 ***************************************************************************/

static void select_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((FilterModule*)data)->select(row); }
static void unselect_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((FilterModule*)data)->unselect(); }
static void up_cb(GtkWidget *w, gpointer data)
{ ((FilterModule*)data)->up(); }
static void down_cb(GtkWidget *w, gpointer data)
{ ((FilterModule*)data)->down(); }
static void permit_cb(GtkWidget *w, gpointer data)
{ ((FilterModule*)data)->permit(); }
static void deny_cb(GtkWidget *w, gpointer data)
{ ((FilterModule*)data)->deny(); }
//static void list_cb(GtkWidget *w, gpointer data)
//{ printf("list\n"); }
//static void import_cb(GtkWidget *w, gpointer data)
//{ printf("import\n"); }
static void delete_cb(GtkWidget *w, gpointer data)
{ ((FilterModule*)data)->del(); }
static void add_cb(GtkWidget *w, gpointer data)
{ ((FilterModule*)data)->add(); }

/***************************************************************************
 FilterModule Class
 ***************************************************************************/

FilterModule::FilterModule(GtkWidget *w)
{
	window = w;
	hbox77_group = NULL;

  vbox58 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox58);

  hbox77 = gtk_hbox_new (FALSE, 18);
  gtk_widget_show (hbox77);
  gtk_box_pack_start (GTK_BOX (vbox58), hbox77, FALSE, TRUE, 0);

  label147 = gtk_label_new ("Default action is to");
  gtk_widget_show (label147);
  gtk_box_pack_start (GTK_BOX (hbox77), label147, FALSE, FALSE, 0);

  radiobutton13 = gtk_radio_button_new_with_label (hbox77_group, "Permit");
  hbox77_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton13));
  gtk_widget_show (radiobutton13);
  gtk_box_pack_start (GTK_BOX (hbox77), radiobutton13, FALSE, FALSE, 0);

  radiobutton14 = gtk_radio_button_new_with_label (hbox77_group, "Deny");
  hbox77_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton14));
  gtk_widget_show (radiobutton14);
  gtk_box_pack_start (GTK_BOX (hbox77), radiobutton14, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (radiobutton14), TRUE);

  hbox78 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox78);
  gtk_box_pack_start (GTK_BOX (vbox58), hbox78, TRUE, TRUE, 0);

  scrolledwindow28 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow28);
  gtk_box_pack_start (GTK_BOX (hbox78), scrolledwindow28, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow28),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  clist25 = gtk_clist_new (3);
  gtk_widget_show (clist25);
  gtk_container_add (GTK_CONTAINER (scrolledwindow28), clist25);
  gtk_clist_set_column_width (GTK_CLIST (clist25), 0, 90);
  gtk_clist_set_column_width (GTK_CLIST (clist25), 1, 90);
  gtk_clist_set_column_width (GTK_CLIST (clist25), 2, 60);
  gtk_clist_column_titles_show (GTK_CLIST (clist25));
	gtk_signal_connect(GTK_OBJECT(clist25), "select_row",
		GTK_SIGNAL_FUNC(select_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist25), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_cb), this);

  label149 = gtk_label_new ("Start Address");
  gtk_widget_show (label149);
  gtk_clist_set_column_widget (GTK_CLIST (clist25), 0, label149);

  label150 = gtk_label_new ("End Address");
  gtk_widget_show (label150);
  gtk_clist_set_column_widget (GTK_CLIST (clist25), 1, label150);

  label151 = gtk_label_new ("Action");
  gtk_widget_show (label151);
  gtk_clist_set_column_widget (GTK_CLIST (clist25), 2, label151);

  vbox55 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox55);
  gtk_box_pack_start (GTK_BOX (hbox78), vbox55, FALSE, TRUE, 0);

  button72 = gtk_button_new ();
  gtk_widget_show (button72);
  gtk_box_pack_start (GTK_BOX (vbox55), button72, FALSE, FALSE, 0);
  gtk_widget_set_usize (button72, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button72), "clicked",
		GTK_SIGNAL_FUNC (up_cb), this);

  pixmap14 = create_pixmap (window, sup_xpm);
  gtk_widget_show (pixmap14);
  gtk_container_add (GTK_CONTAINER (button72), pixmap14);

  button73 = gtk_button_new ();
  gtk_widget_show (button73);
  gtk_box_pack_start (GTK_BOX (vbox55), button73, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button73), "clicked",
		GTK_SIGNAL_FUNC (down_cb), this);

  pixmap15 = create_pixmap (window, sdown_xpm);
  gtk_widget_show (pixmap15);
  gtk_container_add (GTK_CONTAINER (button73), pixmap15);

  button74 = gtk_button_new_with_label ("Permit");
  gtk_widget_show (button74);
  gtk_box_pack_start (GTK_BOX (vbox55), button74, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button74), "clicked",
		GTK_SIGNAL_FUNC (permit_cb), this);

  button75 = gtk_button_new_with_label ("Deny");
  gtk_widget_show (button75);
  gtk_box_pack_start (GTK_BOX (vbox55), button75, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button75), "clicked",
		GTK_SIGNAL_FUNC (deny_cb), this);

  button78 = gtk_button_new_with_label ("Delete");
  gtk_widget_show (button78);
  gtk_box_pack_end (GTK_BOX (vbox55), button78, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button78), "clicked",
		GTK_SIGNAL_FUNC (delete_cb), this);

/*
  button76 = gtk_button_new_with_label ("List");
  gtk_widget_show (button76);
  gtk_box_pack_end (GTK_BOX (vbox55), button76, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button76), "clicked",
		GTK_SIGNAL_FUNC (list_cb), this);

  button77 = gtk_button_new_with_label ("Import");
  gtk_widget_show (button77);
  gtk_box_pack_end (GTK_BOX (vbox55), button77, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button77), "clicked",
		GTK_SIGNAL_FUNC (import_cb), this);
*/

  hbox79 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox79);
  gtk_box_pack_start (GTK_BOX (vbox58), hbox79, FALSE, TRUE, 0);

  entry34 = gtk_entry_new ();
  gtk_widget_show (entry34);
  gtk_box_pack_start (GTK_BOX (hbox79), entry34, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(entry34), "activate",
		GTK_SIGNAL_FUNC(add_cb), this);

  button79 = gtk_button_new_with_label ("Add");
  gtk_widget_show (button79);
  gtk_box_pack_start (GTK_BOX (hbox79), button79, FALSE, FALSE, 0);
  gtk_widget_set_usize (button79, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button79), "clicked",
		GTK_SIGNAL_FUNC (add_cb), this);

	unselect();
}

FilterModule::~FilterModule()
{
}

void FilterModule::select(int r)
{
	gchar *text;

	row = r;
	if (row>0)
		gtk_widget_set_sensitive(button72, TRUE);	// up
	if (row<GTK_CLIST(clist25)->rows-1)
		gtk_widget_set_sensitive(button73, TRUE);	// down
	gtk_clist_get_text(GTK_CLIST(clist25), r, 2, &text);
	if (strcmp(text, "Deny")==0)
		gtk_widget_set_sensitive(button74, TRUE);	// permit
	if (strcmp(text, "Permit")==0)
		gtk_widget_set_sensitive(button75, TRUE);	// deny
	gtk_widget_set_sensitive(button78, TRUE);	// delete
}

void FilterModule::unselect()
{
	gtk_widget_set_sensitive(button72, FALSE);	// up
	gtk_widget_set_sensitive(button73, FALSE);	// down
	gtk_widget_set_sensitive(button74, FALSE);	// permit
	gtk_widget_set_sensitive(button75, FALSE);	// deny
	gtk_widget_set_sensitive(button78, FALSE);	// delete
	row = -1;
}

void FilterModule::up()
{
	if (row>0)
	{
		gtk_clist_swap_rows(GTK_CLIST(clist25), row, row-1);
		row--;
		gtk_widget_set_sensitive(button72, FALSE);	// up
		gtk_widget_set_sensitive(button73, FALSE);	// down
		select(row);
	}
}

void FilterModule::down()
{
	if (row<GTK_CLIST(clist25)->rows-1)
	{
		gtk_clist_swap_rows(GTK_CLIST(clist25), row, row+1);
		row++;
		gtk_widget_set_sensitive(button72, FALSE);	// up
		gtk_widget_set_sensitive(button73, FALSE);	// down
		select(row);
	}
}

void FilterModule::permit()
{
	if (row==-1) return;
	gtk_clist_set_text (GTK_CLIST(clist25), row, 2, "Permit");
	gtk_widget_set_sensitive(button74, FALSE);	// permit
	gtk_widget_set_sensitive(button75, TRUE);	// deny
}

void FilterModule::deny()
{
	if (row==-1) return;
	gtk_clist_set_text (GTK_CLIST(clist25), row, 2, "Deny");
	gtk_widget_set_sensitive(button74, TRUE);	// permit
	gtk_widget_set_sensitive(button75, FALSE);	// deny
}

void FilterModule::del()
{
	if (row==-1) return;
	gtk_clist_remove(GTK_CLIST(clist25), row);
	row = -1;
}

void FilterModule::add()
{
	FilterItem fitem;
	gchar *data[3];
	int r;

	fitem.convertString(gtk_entry_get_text(GTK_ENTRY(entry34)));
	data[0] = strdup(fitem.ip1.getstring());
	data[1] = strdup(fitem.ip2.getstring());
	data[2] = strdup((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radiobutton13)))?"Deny":"Permit");
	r = gtk_clist_append(GTK_CLIST(clist25), data);
	delete data[0]; delete data[1]; delete data[2];
	if ((r>1) && (row>-1))
		gtk_widget_set_sensitive(button73, TRUE);	// down
}

void FilterModule::copytowidgets(Filter *flt)
{
	Link *ln;
	FilterItem *f;
	gchar *data[3];

	row = -1;

	gtk_clist_clear(GTK_CLIST(clist25));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (radiobutton14), TRUE);

	if (flt==NULL) return;

	if (flt->default_action=='p')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (radiobutton13), TRUE);

	for (ln=flt->filter_items.get_head(); ln!=NULL; ln=ln->next)
	{
		f = (FilterItem*) ln->item;
		data[0] = strdup(f->ip1.getstring());
		data[1] = strdup(f->ip2.getstring());
		data[2] = strdup((f->action=='p')?"Permit":"Deny");
		gtk_clist_append(GTK_CLIST(clist25), data);
		delete data[0]; delete data[1]; delete data[2];
	}
}

void FilterModule::copyfromwidgets(Filter *f)
{
	FilterItem *fitem;
	gchar *data;
	int i, num;

	f->default_action = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radiobutton13)))?'p':'d';
	num = GTK_CLIST(clist25)->rows;
	for (i=0; i<num; i++)
	{
		fitem = new FilterItem();
		gtk_clist_get_text(GTK_CLIST(clist25), i, 0, &data);
		fitem->ip1.setstring(data);
		gtk_clist_get_text(GTK_CLIST(clist25), i, 1, &data);
		fitem->ip2.setstring(data);
		gtk_clist_get_text(GTK_CLIST(clist25), i, 2, &data);
		fitem->action = (strcmp(data, "Permit")==0)?'p':'d';
		f->filter_items.tag(fitem);
	}
}

