/* monitor.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gnome.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "monitor.h"
#include "discover.h"
#include "diagram.h"
#include "preferences.h"
#include "sessionlist.h"
#include "message.h"
#include "features.h"
#include "main.h"
#include "gui.h"
#include "data.h"
#include "print.h"
#include "stats.h"
#include "device.h"
#include "devicesidebar.h"
#include "searchsidebar.h"
#include "viewsidebar.h"
#include "filtersidebar.h"
#include "session.h"
#include "subnets.h"
#include "tasklist.h"

#include "licon.xpm"
#include "ssummary.xpm"
#include "srouter.xpm"
#include "ssearch.xpm"
#include "sview.xpm"
#include "smachine.xpm"
#include "scloud.xpm"
#include "sswitch.xpm"
#include "ssubnet.xpm"
#include "sping.xpm"
#include "stopology.xpm"
#include "sdiscover.xpm"

const gchar *authors[] = { "Paul Coates <Paul.Coates@ncl.ac.uk>", NULL };

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((Monitor*)data)->main->quit();
	return (TRUE);
}


// File Callbacks

static void new_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->session->new_session(1); }
static void open_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->session->new_session(2); }
static void save_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->session->have_title(); }
static void saveas_cb(GtkWidget *w, gpointer data)
{
	Monitor *m = (Monitor*) data;
	m->main->session->save_session_list.init(m->main->session->datadir);
	m->main->session->save_session_list.show();
}
static void print_cb(GtkWidget *w, gpointer data)
{
//	gnome_triggers_do("Nomad", "pingsnd",  NULL);
}
static void exit_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->quit(); }


// View Callbacks



// Settings Callbacks

static void sidepanel_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->sidepanel(); }
static void statusbar_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->statbar(); }
static void labels_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->labels(); }
static void colour_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->colour(); }
static void default_cb(GtkWidget *w, gpointer data)
{
	Monitor *m = (Monitor*) data;
	gtk_widget_hide(m->opengl->mainmenu);
	gtk_widget_show(m->viewer->mainmenu);
	gtk_notebook_set_page(GTK_NOTEBOOK (m->notebook2), 0);
}
static void opengl_cb(GtkWidget *w, gpointer data)
{
	Monitor *m = (Monitor*) data;
	gtk_widget_hide(m->viewer->mainmenu);
	gtk_widget_show(m->opengl->mainmenu);
	gtk_notebook_set_page(GTK_NOTEBOOK (m->notebook2), 1);
}
static void preferences_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->gui->preferences->show(); }


// Help Callbacks

static void homepage_cb(GtkWidget *w, gpointer data)
{ gnome_url_show("http://netmon.ncl.ac.uk/"); }
static void manual_cb(GtkWidget *w, gpointer data)
{ gnome_help_goto(NULL, "ghelp:nomad"); }


static void task_cb(GtkWidget *w, gpointer data)
{
	((Monitor*)data)->main->gui->tasklist->show();
}



static void about_cb(GtkWidget *w, gpointer data)
{ 
	static GtkWidget *dialog = NULL;
	GtkWidget *app;

	app = ((Monitor*)data)->mainwindow;

	if (dialog != NULL)
	{
		g_assert(GTK_WIDGET_REALIZED(dialog));
		gdk_window_show(dialog->window);
		gdk_window_raise(dialog->window);
	}else
	{
		gchar *fname = user_file("logo.png");
		dialog = gnome_about_new(_("Nomad"), VERSION,
			"Copyright (C) 2001 Newcastle University",
			authors, _("Network Mapping & Monitoring Tool"), fname);
		g_free(fname);

		gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog);
		gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
		gtk_widget_show(dialog);
	}
}

static GnomeUIInfo file1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New"), NULL, new_cb, NULL),
  GNOMEUIINFO_MENU_OPEN_ITEM (open_cb, NULL),
  GNOMEUIINFO_MENU_SAVE_ITEM (save_cb, NULL),
  GNOMEUIINFO_MENU_SAVE_AS_ITEM (saveas_cb, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_PRINT_ITEM (print_cb, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM (exit_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo view1_menu_uiinfo[] =
{
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("_Task List"), NULL,
    (gpointer) task_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo default1_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("_Default"), NULL,
    (gpointer) default_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_OpenGL"), NULL,
    (gpointer) opengl_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo viewer1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_RADIOITEMS, NULL, NULL, default1_uiinfo,
    NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 0,
    (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo settings1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_TOGGLEITEM, N_("_Side Panel"), NULL,
    (gpointer) sidepanel_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_TOGGLEITEM, N_("S_tatus Bar"), NULL,
    (gpointer) statusbar_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_TOGGLEITEM, N_("_Labels"), NULL,
    (gpointer) labels_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_TOGGLEITEM, N_("_Coloured Links"), NULL,
    (gpointer) colour_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("_Viewer"), NULL,
    viewer1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_PREFERENCES_ITEM (preferences_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo help1_menu_uiinfo[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Homepage"), NULL,
		(gpointer) homepage_cb, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
		0, (GdkModifierType) 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Manual"), NULL,
		(gpointer) manual_cb, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN,
		0, (GdkModifierType) 0, NULL
	},
	GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menubar1_uiinfo[] =
{
  GNOMEUIINFO_MENU_FILE_TREE (file1_menu_uiinfo),
  GNOMEUIINFO_MENU_VIEW_TREE (view1_menu_uiinfo),
  GNOMEUIINFO_MENU_SETTINGS_TREE (settings1_menu_uiinfo),
  GNOMEUIINFO_MENU_HELP_TREE (help1_menu_uiinfo),
  GNOMEUIINFO_END
};



/*
static void subnets_cb(GtkWidget *w, gpointer data)
{
	Monitor* m = (Monitor*) data;
	m->main->gui->subnets->show();
}

static void discover_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->gui->discover->show(); }
*/


/***************************************************************************
 Monitor Class
 ***************************************************************************/

Monitor::Monitor(Main *m)
{
	GtkWidget *vbox77;
	GtkWidget *separator;
	gchar *str;

	main = m;

	// set default preferences
	side_panel = '1';
	status_bar = '1';
	which_viewer = '0';
	str = gnome_config_get_string("/nomad/settings/sidepanel=1");
	if (str!=NULL) side_panel = str[0]; g_free(str);
	str = gnome_config_get_string("/nomad/settings/statusbar=1");
	if (str!=NULL) status_bar = str[0]; g_free(str);
	str = gnome_config_get_string("/nomad/settings/viewer=0");
	if (str!=NULL) which_viewer = str[0]; g_free(str);

  tooltips = gtk_tooltips_new ();

  mainwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (mainwindow), "Nomad");
	gtk_widget_set_usize(mainwindow, 640, 480);
	gtk_widget_realize(mainwindow);
	window_icon(mainwindow, licon_xpm);
	gdk_window_set_icon_name(mainwindow->window, "Nomad");
	gtk_signal_connect(GTK_OBJECT(mainwindow), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(mainwindow), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

  pixmap[0] = create_pixmap(mainwindow, ssummary_xpm);
  pixmap[1] = create_pixmap(mainwindow, srouter_xpm);
  pixmap[2] = create_pixmap(mainwindow, ssearch_xpm);
  pixmap[3] = create_pixmap(mainwindow, sview_xpm);
  pixmap[4] = create_pixmap(mainwindow, sswitch_xpm);
  pixmap[5] = create_pixmap(mainwindow, smachine_xpm);
  pixmap[6] = create_pixmap(mainwindow, scloud_xpm);
  pixmap[7] = create_pixmap(mainwindow, ssubnet_xpm);
  pixmap[8] = create_pixmap(mainwindow, srouter_xpm);

	taskpix[0] = create_pixmap(mainwindow, sping_xpm);
	taskpix[1] = create_pixmap(mainwindow, stopology_xpm);
	taskpix[2] = create_pixmap(mainwindow, sdiscover_xpm);

	viewer = new Viewer(mainwindow, main);
	opengl = new OpenGL(mainwindow, main);
	devicesidebar = new DeviceSideBar(main);
	searchsidebar = new SearchSideBar(main);
	viewsidebar = new ViewSideBar(main);
	filtersidebar = new FilterSideBar(main);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (mainwindow), vbox1);

  menubar = gtk_menu_bar_new();
  gtk_widget_show(menubar);
  gtk_box_pack_start(GTK_BOX(vbox1), menubar, FALSE, FALSE, 0);
  gnome_app_fill_menu_with_data (GTK_MENU_SHELL (menubar), menubar1_uiinfo,
		NULL, FALSE, 0, this);

	hpaned1 = gtk_hpaned_new ();
  gtk_widget_show (hpaned1);
  gtk_box_pack_start (GTK_BOX (vbox1), hpaned1, TRUE, TRUE, 0);
  gtk_paned_set_gutter_size (GTK_PANED (hpaned1), 10);
	gtk_paned_set_handle_size (GTK_PANED (hpaned1), 10);
  gtk_paned_set_position (GTK_PANED (hpaned1), 160);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_paned_pack1 (GTK_PANED (hpaned1), notebook1, FALSE, TRUE);

  gtk_container_add (GTK_CONTAINER (notebook1), devicesidebar->vbox67);

  vbox77 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox77);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox77);
  gtk_container_add (GTK_CONTAINER (notebook1), searchsidebar->vbox12);

	gtk_box_pack_start (GTK_BOX (vbox77), viewsidebar->vbox67, TRUE, TRUE, 0);
  separator = gtk_hseparator_new ();
  gtk_widget_show (separator);
  gtk_box_pack_start (GTK_BOX (vbox77), separator, FALSE, TRUE, 2);
	gtk_box_pack_start (GTK_BOX (vbox77), filtersidebar->vbox56, TRUE, TRUE, 0);

	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), pixmap[1]);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), pixmap[3]);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), pixmap[2]);

	notebook2 = gtk_notebook_new ();
  gtk_widget_show (notebook2);
  gtk_paned_pack2 (GTK_PANED (hpaned1), notebook2, TRUE, FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK (notebook2), FALSE);
	gtk_notebook_set_show_border(GTK_NOTEBOOK (notebook2), FALSE);

  gtk_container_add (GTK_CONTAINER (notebook2), viewer->vbox3);
  gtk_container_add (GTK_CONTAINER (notebook2), opengl->vbox3);

	gtk_menu_bar_insert(GTK_MENU_BAR(menubar), viewer->mainmenu, 1);
	gtk_menu_bar_insert(GTK_MENU_BAR(menubar), opengl->mainmenu, 1);

	create_statusbar(vbox1);

	separatorbar = gtk_hseparator_new();
  gtk_widget_show(separatorbar);
  gtk_box_pack_end(GTK_BOX(vbox1), separatorbar, FALSE, TRUE, 0);

  gtk_object_set_data (GTK_OBJECT (mainwindow), "tooltips", tooltips);

// Apply Preferences

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[0].widget),
		(side_panel=='1')?TRUE:FALSE);

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[1].widget),
		(status_bar=='1')?TRUE:FALSE);

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[2].widget),
		(viewer->diagram->showlabels=='1')?TRUE:FALSE);

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[3].widget),
		(viewer->diagram->showcolourlines=='1')?TRUE:FALSE);

	if (which_viewer=='0')
	{
		gtk_check_menu_item_set_active(
			GTK_CHECK_MENU_ITEM(default1_uiinfo[0].widget), TRUE);
		gtk_widget_hide(opengl->mainmenu);
	}else
	if (which_viewer=='1')
	{
		gtk_check_menu_item_set_active(
			GTK_CHECK_MENU_ITEM(default1_uiinfo[1].widget), TRUE);
		gtk_widget_hide(viewer->mainmenu);
	}
}

Monitor::~Monitor()
{
	if (viewer!=NULL) delete viewer;
	if (devicesidebar!=NULL) delete devicesidebar;
	if (searchsidebar!=NULL) delete searchsidebar;
	if (viewsidebar!=NULL) delete viewsidebar;
}

void Monitor::show()
{
	gtk_widget_show(mainwindow);
	gdk_window_raise(mainwindow->window);
}

void Monitor::hide()
{
	gtk_widget_hide(mainwindow);
}

void Monitor::sidepanel()
{
	if (GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[0].widget)->active)
	{
		gtk_widget_show(hpaned1);
		gtk_widget_ref(notebook2);
		if (notebook2->parent == vbox1)
			gtk_container_remove(GTK_CONTAINER(vbox1), notebook2);
		if (notebook2->parent == NULL)
			gtk_container_add(GTK_CONTAINER(hpaned1), notebook2);
		gtk_widget_unref(notebook2);
	}else
	{
		gtk_widget_hide(hpaned1);
		gtk_widget_ref(notebook2);
		if (notebook2->parent == hpaned1)
			gtk_container_remove(GTK_CONTAINER(hpaned1), notebook2);
		if (notebook2->parent == NULL)
			gtk_container_add(GTK_CONTAINER(vbox1), notebook2);
		gtk_widget_unref(notebook2);
	}
}

void Monitor::statbar()
{
	if (GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[1].widget)->active)
	{
		gtk_widget_show(statusbar);
		gtk_widget_show(separatorbar);
	}else
	{
		gtk_widget_hide(statusbar);
		gtk_widget_hide(separatorbar);
	}
}

void Monitor::labels()
{
	viewer->diagram->showlabels =
		(GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[2].widget)->active)?'1':'0';
	viewer->draw();
}

void Monitor::colour()
{
	viewer->diagram->showcolourlines =
		(GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[3].widget)->active)?'1':'0';
	viewer->draw();
}

void Monitor::create_statusbar(GtkWidget *vbox)
{
	GtkWidget *button;
	GtkWidget *frame;
	GtkWidget *hbox;

	statusbar = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (statusbar);
  gtk_box_pack_end (GTK_BOX (vbox), statusbar, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (statusbar), 2);

	message = gtk_label_new("");
	gtk_widget_show (message);
	gtk_box_pack_start (GTK_BOX (statusbar), message, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (message), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (message), 2, 0);

  frame = gtk_frame_new (NULL);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (statusbar), frame, FALSE, TRUE, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);

	hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_container_add (GTK_CONTAINER (frame), hbox);

	counter[0] = gtk_label_new ("0");
	counter[1] = gtk_label_new ("0");
	counter[2] = gtk_label_new ("0");
	counter[3] = gtk_label_new ("0");
	counter[4] = gtk_label_new ("0");

	statusbar_variable(hbox, counter[0], "Routers", pixmap[8]);
	statusbar_variable(hbox, counter[1], "Switches", pixmap[4]);
	statusbar_variable(hbox, counter[2], "Hosts", pixmap[5]);
	statusbar_variable(hbox, counter[3], "Network Clouds", pixmap[6]);
	statusbar_variable(hbox, counter[4], "Subnets", pixmap[7]);

	frame = gtk_frame_new (NULL);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (statusbar), frame, FALSE, TRUE, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);

  button = gtk_button_new ();
  gtk_widget_show (button);
  gtk_container_add (GTK_CONTAINER (frame), button);
  gtk_tooltips_set_tip (tooltips, button, "Task List", NULL);
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
	gtk_signal_connect (GTK_OBJECT (button), "pressed",
		GTK_SIGNAL_FUNC (task_cb), this);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_container_add (GTK_CONTAINER (button), hbox);

  gtk_box_pack_start (GTK_BOX (hbox), taskpix[0], FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), taskpix[1], FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), taskpix[2], FALSE, TRUE, 0);

	gtk_widget_set_sensitive(taskpix[0], FALSE);
	gtk_widget_set_sensitive(taskpix[1], FALSE);
	gtk_widget_set_sensitive(taskpix[2], FALSE);
}

void Monitor::statusbar_variable(GtkWidget *box, GtkWidget *label, char *tip, GtkWidget *pix)
{
	GtkWidget *eventbox;
	GtkWidget *hbox;

	eventbox = gtk_event_box_new ();
  gtk_widget_show (eventbox);
  gtk_box_pack_start (GTK_BOX (box), eventbox, FALSE, TRUE, 2);
  gtk_tooltips_set_tip (tooltips, eventbox, tip, NULL);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox);
  gtk_container_add (GTK_CONTAINER (eventbox), hbox);

  gtk_box_pack_start (GTK_BOX (hbox), pix, FALSE, TRUE, 0);

  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
}

void Monitor::statusbar_update()
{
	char buf[64];
	Link *ln;
	Device *d;
	int ns=0, nr=0;
	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (d->type=='s') ns++; else
		if (d->type=='r') nr++;
	}

	if (main->session->data!=NULL)
	{
		sprintf(buf, "%d", nr);
		gtk_label_set_text(GTK_LABEL(counter[0]), buf);
		sprintf(buf, "%d", ns);
		gtk_label_set_text(GTK_LABEL(counter[1]), buf);
		sprintf(buf, "%d", main->session->data->machines.size());
		gtk_label_set_text(GTK_LABEL(counter[2]), buf);
		sprintf(buf, "%d", main->session->data->networks.size());
		gtk_label_set_text(GTK_LABEL(counter[3]), buf);
		sprintf(buf, "%d", main->session->data->subnets.size());
		gtk_label_set_text(GTK_LABEL(counter[4]), buf);
	}
}


/*
void Preferences::load_settings()
{
	gint x, y, width, height;

	gdk_window_get_root_origin (main->gui->monitor->mainwindow, &x, &y);
	gdk_window_get_size (main->gui->monitor->mainwindow, &width, &height);

	state_push_prefix (name);
	gnome_config_set_int ("_x=", x);
	gnome_config_set_int ("_y=", y);
	gnome_config_set_int ("_width=", width);
	gnome_config_set_int ("_height=", height);
	gnome_config_pop_prefix ();
	gnome_config_sync ();
}

void
state_load_window_state (GtkWidget *window, const gchar *name,
			int defwidth, int defheight)
{
	GString *buf;
	gint x, y, width, height;
	
	buf = g_string_sized_new (16);
	
	state_push_prefix (name);

	x = gnome_config_get_int ("_x=-1");
	y = gnome_config_get_int ("_y=-1");
	if (!(x == -1 && y == -1))
		gtk_widget_set_uposition (window, x, y);

	g_string_sprintf (buf, "_width=%i", defwidth);
	width = gnome_config_get_int (buf->str);
	g_string_sprintf (buf, "_height=%i", defheight);
	height = gnome_config_get_int (buf->str);
	if (!(width == -1 && height == -1))
		gtk_window_set_default_size (GTK_WINDOW (window),
					     width, height);

	g_string_free (buf, TRUE);
	gnome_config_pop_prefix ();
}

void Preferences::save_settings()
{
	gchar *str;

	gnome_config_set_string("/nomad/settings/sidepanel", main->gui->monitor->side_panel);
	g_free(str);
}
*/

