/* parser.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <arpa/inet.h>
#include <pwd.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include <stdio.h>
#include <netdb.h>
#include "parser.h"
#include "scheduler.h"
#include "discover.h"
#include "preferences.h"
#include "process.h"
#include "task.h"
#include "main.h"
#include "session.h"
#include "data.h"
#include "misc.h"
#include "gui.h"

Parser::Parser(Main *m)
{
	main = m;
}

Parser::~Parser()
{
}

//////////////////////////////////////////////////////////////////////////////
//
// Code to parse a topology file
//
//////////////////////////////////////////////////////////////////////////////

void Parser::create_placeholder(char *ipstr)
{
	Device *d;
	struct hostent *h;
	struct in_addr addr;
	IPAddress tip;

	tip.setstring(ipstr);

	if (inet_aton(tip.getstring(), &addr) == 0)
	{
		printf("Invalid IP address\n");
		return;
	}

	h = gethostbyaddr((const char *) &addr.s_addr, sizeof(addr), AF_INET);
	if (!h)
	{
		printf("Reverse Lookup failed\n");
		return;
	}

	if (!tip.zero())
	{
		// see if device already exists
		d = main->session->data->find_ip_on_device(tip);
		if (d==NULL)
		{
			d = new Device();
			d->pingname.set(tip);
			strcpy(d->name, h->h_name);
			d->type = 'u';
			main->session->data->devices.tag(d);
			main->gui->update_views();
			main->gui->update_lists();
		}
	}
}

void Parser::getDescr(Device *d, char *str, int val)
{
	FILE *f;
	char comm[64], tmpbuf[64], lst[256], buff[256];
	gchar *fname;
	char type;
	int mask, serv, len, i, pos, status;

	printf("START\n");
	fname = user_file("device-types");
	if (fname!=NULL)
	{
		f = fopen(fname, "r");
		if (f)
		{
			while (fgets(buff, 1023, f))
			{
				if ((buff[0]!='\n') && (buff[0]!='#'))
				{
					if ((sscanf(buff, "%c \"%[^\"]\" %d %d %[^\n]\n", &type, comm, &mask, &serv, lst)==5) || (sscanf(buff, "%c \"%[^\"]\" %d %d\n", &type, comm, &mask, &serv)==4))
					{
						status = 1;

						if ((val&mask)!=serv) status = 0;
						

//						printf("%s %s %d %d\n", ((type=='r')?"Router":"Switch"), comm, mask, serv);

						len = strlen(lst);
						pos = 0;
						for (i=0; i<len; i++)
						{
							if (lst[i]!=' ')
							{
								tmpbuf[pos] = lst[i];
								pos++;
							}else
							{
								tmpbuf[pos] = '\0';
								if (!lower_contains(str, tmpbuf)) status = 0;
//								printf("PARAM %s\n", tmpbuf);
								pos = 0;
							}
						}
						if (pos>0)
						{
							tmpbuf[pos] = '\0';
							if (!lower_contains(str, tmpbuf)) status = 0;
//							printf("PARAMEND %s\n", tmpbuf);
						}

						if (status==1)
						{
							strcpy(d->descr, comm);
							d->type = type;
						}
					}
				}
/*
					if (((s&4)==4) && (lower_contains(dscr, "cisco")))
					{
						strcpy(tmpdev->descr, "Cisco Router"); tmpdev->type = 'r';
					}else
					if (((s&6)==2) && (lower_contains(dscr, "cisco")))
					{
						strcpy(tmpdev->descr, "Cisco Switch"); tmpdev->type = 's';
					}else
					if (((s&6)==2) && (lower_contains(dscr, "hp")) &&
							(lower_contains(dscr, "switch")) &&
							(lower_contains(dscr, "procurve")) &&
							(lower_contains(dscr, "2524")))
					{
						strcpy(tmpdev->descr, "HP ProCurve 2524 Switch");
						tmpdev->type = 's';
					}else
					if ((s&6)==6)
					{
						tmpdev->type = 'r';
					}else
					if ((s&6)==2)
					{
						tmpdev->type = 's';
					}
*/
			}
		}
	}
	printf("STOP\n");
}

void Parser::parse(char *fnameparam, int hop)
{
	FILE *f;
	char buf[256], var[256], val[256], fname[256], dscr[256];
	int s;
	Device *d;
	IPAddress tip;
	struct in_addr addr;
	struct hostent *h;

	sscanf(fnameparam, "%[^ ] %[^\n]", fname, buf);
	printf("PARSING RESULT %s\n", fname);

	sprintf(buf, "Parsing %s\n", fname);
	main->gui->discover->discover_message(buf);

	sprintf(buf, "%s/topology/%s", main->session->datadir, fname);
	f = fopen(buf, "r");
	if (!f) return;

	// find the device that we created earlier
	tip.setstring(fname);
	tmpdev = main->session->data->find_ip_on_device(tip);
	if (tmpdev==NULL) return;
	// if not a placeholder device must be real
	// so we should merge later, else just use device
	if (tmpdev->type!='u')
	{
		tmpdev = new Device();
		main->session->data->devices.tag(tmpdev);
	}

	tmpdev->pingname.setstring(fname);
	strcpy(tmpdev->name, fname);
	if (inet_aton(fname, &addr) != 0)
	{
		h = gethostbyaddr((const char *) &addr.s_addr, sizeof(addr), AF_INET);
		if (h) strcpy(tmpdev->name, h->h_name);
	}

	fscanf(f, "%[^\n]\n", buf);
	tmpdev->last_seen.setstring(buf);
	fscanf(f, "%[^\n]\n", tmpdev->community);

	subnets.empty();
	while(fscanf(f, "%[^\n]\n", buf)==1)
	{
		if (sscanf(buf, "%[^ ] %[^\n]", var, val)==2)
		{
			if (strcmp(".iso.3.6.1.2.1.1.1.0", var)==0)
			{
				val[strlen(val)-1]='\0';
				strcpy(dscr, val+1);
			}else
			if (strcmp(".iso.3.6.1.2.1.1.7.0", var)==0)
			{
				if (sscanf(val, "%d", &s)==1)
				{
					getDescr(tmpdev, dscr, s);
				}
			}else
			if (strcmp(".iso.3.6.1.2.1.1.5.0", var)==0)
			{
				// system name overrides DNS name because
				// of the way auto discovery works
				val[strlen(val)-1]='\0';
				strcpy(tmpdev->name, val+1);
			}else
			if (strcmp(".iso.3.6.1.2.1.1.4.0", var)==0)
			{
				val[strlen(val)-1]='\0';
				strcpy(tmpdev->contact, val+1);
			}else
			if (strcmp(".iso.3.6.1.2.1.1.6.0", var)==0)
			{
				val[strlen(val)-1]='\0';
				strcpy(tmpdev->location, val+1);
			}else
			if (starts_with(".iso.3.6.1.2.1.2.2.1", var))
			{
				add_to_interface(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.31.1.1.1", var))
			{
				add_to_interface_extended(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.4.20.1", var))
			{
				add_to_ips(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.4.21.1", var))
			{
				add_to_route(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.4.22.1.2", var))
			{
				add_to_address(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.17.1.4.1.2", var))
			{
				add_to_bridge_map(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.17", var))
			{
				add_to_bridge(var, val);
			}
		}
	}

	// done parsing file so now build the rest

	build_subnets(hop);

	// empty subnets here

	// delete original device
	d = main->session->data->find_ip_on_device(tmpdev->pingname);
	if ((d!=NULL) && (tmpdev!=d))
	{
		// copy coordinates
		tmpdev->x = d->x;
		tmpdev->y = d->y;
		tmpdev->show = d->show;
		tmpdev->selected = d->selected;
		tmpdev->status = d->status;
		tmpdev->grab_top = d->grab_top;
		tmpdev->grab_stats = d->grab_stats;
		// copy network coordinates

		// delete device
		main->session->data->del_device(d);
	}
	main->gui->update_views();
	main->gui->update_lists();
}

void Parser::add_to_interface(char *var, char *val)
{
	char vartype[256], junk[256];
	Interface *in, *tin;
	Network *n;
	Link *ln;
	unsigned long s;
	int j, ifnum;

	if (sscanf(var, ".iso.3.6.1.2.1.2.2.1.%[^.].%d", vartype, &ifnum)!=2)
		return;

	tin = NULL;
	for (ln=tmpdev->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in->index == ifnum) tin = in;
	}

	if (tin == NULL)
	{
		tin = new Interface();
		tin->index = ifnum;
		tin->device = tmpdev;
		n = new Network();
		n->interfaces.tag(tin);
		tin->network = n;
		main->session->data->networks.tag(n);
		tmpdev->interfaces.tag(tin);
	}

	if ((strcmp("5", vartype)==0) && (sscanf(val, "%lu", &s)==1))
		tin->speed = s;
	else
	if ((strcmp("2", vartype)==0) && (sscanf(val, "\"%[^\"]\"", junk)==1))
		strcpy(tin->descr, junk);
	else
	if ((strcmp("3", vartype)==0) && (sscanf(val, "%d", &j)==1))
		tin->type = j;
	else
	if ((strcmp("8", vartype)==0) && (sscanf(val, "%[^\n]", junk)==1))
		tin->status = junk[0];
	else
	if (strcmp("6", vartype)==0)
		tin->mac.setstringsnmp(val);
}

void Parser::add_to_interface_extended(char *var, char *val)
{
	char vartype[256], junk[256];
	Interface *in, *tin;
	Network *n;
	Link *ln;
	int ifnum;

	if (sscanf(var, ".iso.3.6.1.2.1.31.1.1.1.%[^.].%d", vartype, &ifnum)!=2)
		return;

	tin = NULL;
	for (ln=tmpdev->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in->index == ifnum) tin = in;
	}

	if (tin == NULL)
	{
		tin = new Interface();
		tin->index = ifnum;
		tin->device = tmpdev;
		n = new Network();
		n->interfaces.tag(tin);
		tin->network = n;
		main->session->data->networks.tag(n);
		tmpdev->interfaces.tag(tin);
	}

	if (strcmp("1", vartype)==0)
	{
		if ((sscanf(val, "\"%[^\"]\"", junk)==1) && (strlen(junk)>0))
			strcpy(tin->descr, junk);
	}else
	if (strcmp("18", vartype)==0)
	{
		if ((sscanf(val, "\"%[^\"]\"", junk)==1) && (strlen(junk)>0))
			strcpy(tin->comment, junk);
	}
}

void Parser::add_to_ips(char *var, char *val)
{
	char vartype[256], idxval[256];
	Interface *in, *tin;
	Link *ln;
	IPAddress *tip;
	int ifnum;

	if (sscanf(val, "%d", &ifnum)!=1) return;
	if (sscanf(var, ".iso.3.6.1.2.1.4.20.1.%[^.].%[^\n]", vartype, idxval)!=2)
		return;

	tin = NULL;
	for (ln=tmpdev->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in->index == ifnum) tin = in;
	}

	if (tin != NULL)
	{
		if (strcmp("2", vartype)==0)
		{
			tip = new IPAddress();
			tip->setstring(idxval);
			tin->ip_addresses.tag(tip);
		}
	}
}

void Parser::add_to_route(char *var, char *val)
{
	char vartype[256], idxval[256];
	SubnetExtra *sn, *tsn;
	IPAddress tip, tmsk;
	Link *ln;
	int j;

	if (sscanf(var, ".iso.3.6.1.2.1.4.21.1.%[^.].%[^\n]", vartype, idxval)!=2)
		return;

	tip.setstring(idxval);

	tsn = NULL;
	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		sn = (SubnetExtra*) ln->item;
		if (sn->ip.compare(tip)==0) tsn = sn;
	}

	if (tsn == NULL)
	{
		tsn = new SubnetExtra();
		tsn->ip.set(tip);
		tsn->mask = 0;
		tsn->next.empty();
		tsn->index = 0;
		tsn->type = 0;
		subnets.tag(tsn);
	}

	if ((strcmp("2", vartype)==0) && (sscanf(val, "%d", &j)==1))
	{
		tsn->index = j;
	}else
	if (strcmp("11", vartype)==0)
	{
		tmsk.setstring(val);
		tsn->mask = tmsk.mask_value();
	}else
	if (strcmp("7", vartype)==0)
	{
		tsn->next.setstring(val);
	}else
	if ((strcmp("8", vartype)==0) && (sscanf(val, "%d", &j)==1))
	{
		tsn->type = j;
	}
}

void Parser::add_to_address(char *var, char *val)
{
	char vartype[256], junk[256];
	MACAddress tmac;
	Machine *m, *tmpm;
	Interface *in;
	Link *ln;
	int ifnum;

	if (sscanf(var, ".iso.3.6.1.2.1.4.22.1.2.%[^.].%[^\n]", vartype, junk)!=2)
		return;
	if (sscanf(vartype, "%d", &ifnum)!=1) return;

	tmac.setstringsnmp(val);
	tmpm = NULL;
	for (ln=main->session->data->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (m->mac.compare(tmac)==1) tmpm = m;
	}

	if (tmpm == NULL)
	{
		m = new Machine();
		m->mac = tmac;
		for (ln=tmpdev->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			if (in->index == ifnum) in->network->machines.tag(m);
			m->network = in->network;
		}
		main->session->data->machines.tag(m);
	}

	m->ip.setstring(junk);
}

void Parser::add_to_bridge_map(char *var, char *val)
{
	int j, ifnum;
	Port *p, *tmpp;
	Link *ln;

	if (sscanf(var, ".iso.3.6.1.2.1.17.1.4.1.2.%d", &ifnum)!=1) return;
	sscanf(val, "%d", &j);

	tmpp = NULL;
	for (ln=ports.get_head(); ln!=NULL; ln=ln->next)
	{
		p = (Port*) ln->item;
		if (p->index == ifnum) tmpp = p;
	}

	if (tmpp==NULL)
	{
		p = new Port();
		p->index = ifnum;
	}

	p->port = j;
}

void Parser::add_to_bridge(char *var, char *val)
{
	char newmac[64];
	MACAddress tmpmac;
	Machine *m, *tmpm;
//	int i, j, iffound = -1, ifgap = -1;
	Link *ln;

	// a bridge variable is split into 2
	if (sscanf(var, ".iso.3.6.1.2.1.17.4.3.1.2.%[^\n]", newmac)!=1) return;
	tmpmac.setstringsnmp2(newmac);

	tmpm = NULL;
	for (ln=main->session->data->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (m->mac.compare(tmpmac)==1) tmpm = m;
	}


/*
	for (i=0; i<dv[pos].num_macs; i++)
	{
		if (compare_mac(dv[pos].macs[i].mac, vartype)==1) iffound = i;
		if (zero_mac(dv[pos].macs[i].mac) && (ifgap == -1)) ifgap = i;
	}
	if (iffound == -1) // failed to find interface
	{
		if (ifgap != -1) // found a gap
		{
			iffound = ifgap;
			for (i=0; i<6; i++) dv[pos].macs[iffound].mac[i] = vartype[i];
		}else
			return;
	}
	sscanf(val, "%d", &j);
	dv[pos].macs[iffound].port = j;
}

void Topology::port_to_if()
{
	int i, j, tmp;

	for (i=0; i<dv[pos].num_macs; i++)
	{
		tmp = dv[pos].macs[i].port;
		for (j=0; j<num_ports; j++)
		{
			if (tmp == pt[j].index)
				dv[pos].macs[i].port = pt[j].port;
		}
	}
*/



}


//////////////////////////////////////////////////////////////////////////////
//
// Code to merge a parsed device
//
//////////////////////////////////////////////////////////////////////////////

void Parser::build_subnets(int hop)
{
	Interface *in;
	SubnetExtra *r;
	Subnet *s;
	Link *ln, *ln2;
/*
	for (ln2=subnets.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		r = (SubnetExtra*) ln2->item;
		printf("SubnetExtra %s/%d, ", r->ip.getstring(), r->mask);
		printf("%s, %d, %d\n", r->next.getstring(), r->index, r->type);
	}
*/
	for (ln=tmpdev->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		for (ln2=subnets.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			r = (SubnetExtra*) ln2->item;
			if ((r->index == in->index) &&
					(!(r->next.in_range("127.0.0.0", "127.0.0.255"))) &&
					(r->next.zero()==0))
			{
				if (r->type == 3)
				{
					s = new Subnet();
					s->ip.set(r->ip);
					s->mask = r->mask;
					in->network->subnets.tag(s);
					add_subnet(s);
				}else
				if (r->type == 4)
				{
					add_layer3(in->network, r->next, hop);
				}
			}
		}
	}
}

// adds subnet to global list if not already added

void Parser::add_subnet(Subnet *s)
{
	Link *ln;
	Subnet *s2;
	int got=0;

	if (s->mask > 30) return;
	for (ln=main->session->data->subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		s2 = (Subnet*) ln->item;
		if ((s->ip.compare(s2->ip)==0) && (s2->mask == s->mask)) got++;
	}
	if (!got) main->session->data->subnets.tag(s);
}

void Parser::add_layer3(Network *n, IPAddress ip, int hop)
{
	Link *ln, *ln2, *ln3;
//	char buf[1024];
	Device *d;
	Interface *in;
	IPAddress *tip;
	int found;

	found = 0;
	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		for (ln2=d->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			in = (Interface*) ln2->item;
			for (ln3=in->ip_addresses.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				tip = (IPAddress*) ln3->item;
				if (tip->compare(ip)==0)
				{
					if (in->network != n)
					{
						// we need to merge the networks n & in->network
						merge_networks(n, in->network);
//					printf("MERGE NETWORKS: Found match %s, %d\n", d->name, in->index);
					}
					found = 1;
				}
			}
		}
	}

	if ((found==0) && (hop>0))
	{
		// if task not already assigned then
		// topology, priority 3
//		sprintf(buf, "%s %s",
//			ip.getstring(), main->session->options->getCommunityList());
//		main->scheduler->addTask(buf, 1, 3, hop-1);
//		printf("NEW DEVICE: New task (%s, 3, 1, %d)\n", ip.getstring(), hop-1);
	}
}

void Parser::merge_networks(Network *n1, Network *n2)
{
	Link *ln;
	Interface *in;

	if (n1==n2) return;

	// n1 is the new network
	// n2 is the old network
	// we move everything from n2 to n1

	// move over interfaces
	for (ln=n2->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in!=NULL)
		{
			in->network = n1;
			n1->interfaces.tag(in);
		}
	}

	// now move over subnets (avoid duplicates)

	// now move over machines (avoid duplicates)

	main->session->data->networks.del(n2);
	delete n2;
}

//////////////////////////////////////////////////////////////////////




/*
void Parser::subnets()
{
	Interface *in;
	Route *r;
	Subnet *s;
	Link *ln, *ln2;

	for (ln=tmpdev->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		for (ln2=routes.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			r = (Route*) ln2->item;
			if ((r->index == in->index) &&
					(r->type == 3) &&
					(!(r->dest.in_range("127.0.0.0", "127.0.0.255"))) &&
					(r->dest.zero()==0))
			{
				s = new Subnet();
				s->ip.set(r->dest);
				s->mask = r->mask;
				in->network->subnets.tag(s);
				add_subnet(s);
			}
		}
	}
}

void Parser::merge_subnets()
{
	Link *ln, *ln2, *ln3;
	Interface *in;
	Subnet *s, *s2;
	Network *n, *n2;
	int fnd;

	for (ln=tmpdev->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		n = in->network;
		if (n!=NULL)
		{
			if (n->subnets.size()>0)
			{
				ln2 = n->subnets.get_head();
				s = (Subnet*) ln2->item;

				// look for subnet in all other networks
				for (ln2=data->networks.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					n2 = (Network*) ln2->item;
					if ((n2!=NULL) && (n != n2))
					{
						fnd=0;
						for (ln3=n2->subnets.get_head(); ln3!=NULL; ln3=ln3->next)
						{
							s2 = (Subnet*) ln3->item;
							if ((s->ip.compare(s2->ip)==0) && (s->mask == s2->mask)) fnd=1;
						}
						if (fnd==1)
						{
							printf("FOUND MATCH, MERGE\n");
							printf("%s/%d\n", s->ip.getstring(), s->mask);
							printf("%s/%d\n", s2->ip.getstring(), s2->mask);
*/
/*
							// both same so merge the networks
							for (ln5=n->interfaces.get_head(); ln5!=NULL; ln5=ln5->next)
							{
								in = (Interface*) ln5->item;
								n2->interfaces.tag(in);
								in->network = n2;
							}
							ln4 = ln->next;
							newdata.networks.del(n);
							delete n;
							ln = ln4;
*/
/*
						}
					}
				}
			}
		}
	}
}

void Parser::machines()
{
}

void Parser::merge_machines()
{
}
*/

