#include <arpa/inet.h>
#include <netdb.h>
#include <stdlib.h>
#include <string.h>
#include "filteritem.h"

FilterItem::FilterItem()
{
	empty();
}

FilterItem::~FilterItem()
{
}

FilterItem::FilterItem(FilterItem &orig)
{
	ip1 = orig.ip1;
	ip2 = orig.ip2;
	action = orig.action;
}

void FilterItem::operator = (const FilterItem &orig)
{
	ip1 = orig.ip1;
	ip2 = orig.ip2;
	action = orig.action;
}

void FilterItem::empty()
{
	ip1.empty();
	ip2.empty();
	action = 'd';
}

void FilterItem::print()
{
	char tmpbuf[64];
	
	strcpy(tmpbuf, ip2.getstring());
	printf("%s %s %s\n", ip1.getstring(), tmpbuf,
		(action == 'd')?"Deny":"Permit");
}

// machine.domain
// a.b.c.d
// a.b.c.d/24
// a.b.c.d 255.255.0.0
// a.b.c.d e.f.g.h
// a.b.c.d-e.f.g.h

void FilterItem::convertString(char *str)
{
	int a, b, c, d, e, f, g, h;
	char buf[64];
	struct hostent *hst;

	if (strcmp(str, "")!=0)
	{
		if (sscanf(str, "%d.%d.%d.%d-%d.%d.%d.%d",
			&a, &b, &c, &d, &e, &f, &g, &h)==8)
		{
			sprintf(buf, "%d.%d.%d.%d", a, b, c, d);
			ip1.setstring(buf);
			sprintf(buf, "%d.%d.%d.%d", e, f, g, h);
			ip2.setstring(buf);
		}else
		if (sscanf(str, "%d.%d.%d.%d %d.%d.%d.%d",
			&a, &b, &c, &d, &e, &f, &g, &h)==8)
		{
			sprintf(buf, "%d.%d.%d.%d", a, b, c, d);
			ip1.setstring(buf);
			sprintf(buf, "%d.%d.%d.%d", e, f, g, h);
			ip2.setstring(buf);
			if (ip2.valid_mask())
			{
				e = ip2.mask_value();
				f = mask_power(e);
				ip1.make_ip(ip1, e, 0);
				ip2.make_ip(ip1, e, f-1);
			}
		}else
		if (sscanf(str, "%d.%d.%d.%d/%d", &a, &b, &c, &d, &e)==5)
		{
			sprintf(buf, "%d.%d.%d.%d", a, b, c, d);
			ip1.setstring(buf);
			f = mask_power(e);
			ip1.make_ip(ip1, e, 0);
			ip2.make_ip(ip1, e, f-1);
		}else
		if (sscanf(str, "%d.%d.%d.%d", &a, &b, &c, &d)==4)
		{
			ip1.setstring(str);
			ip2.setstring(str);
		}else
		if ((hst=gethostbyname(str))!=NULL)
		{
			ip1.setstring(inet_ntoa(*((struct in_addr *)hst->h_addr)));
			ip2.setstring(inet_ntoa(*((struct in_addr *)hst->h_addr)));
		}
	}
}

void FilterItem::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"FilterItem", NULL);
	xmlSetProp(newxml, (xmlChar*)"ip1", (xmlChar*)ip1.getstring());
	xmlSetProp(newxml, (xmlChar*)"ip2", (xmlChar*)ip2.getstring());
	xmlSetProp(newxml, (xmlChar*)"action",
		(xmlChar*)((action=='d')?"Deny":"Permit"));
}

void FilterItem::loadxml(xmlNodePtr xmlnode)
{
	xmlChar *buf;

	if (!xmlnode->name || strcmp((char*)xmlnode->name, "FilterItem")!=0) return;
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"ip1");
	if (buf!=NULL) { ip1.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"ip2");
	if (buf!=NULL) { ip2.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"action");
	if (buf!=NULL)
	{ action = ((strcmp((char*)buf, "Deny")==0)?'d':'p'); free(buf); }
}

